'use strict';

var _camelCase2 = require('lodash/camelCase');

var _camelCase3 = _interopRequireDefault(_camelCase2);

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (_ref) {
    var t = _ref.types;

    var deduceName = undefined,
        isAnnonClassDecl = undefined,
        isAnnonFunctionDecl = undefined,
        isIdentifierDecl = undefined,
        isLiteralDecl = undefined,
        isNamedClassDecl = undefined,
        isNamedFunctionDecl = undefined,
        isObjectDecl = undefined,
        resolveClashingName = undefined,
        transform = undefined;

    isNamedFunctionDecl = function isNamedFunctionDecl(declaration) {
        return declaration.type === 'FunctionDeclaration' && declaration.id && Boolean(declaration.id.name);
    };

    isIdentifierDecl = function isIdentifierDecl(declaration) {
        return declaration.type === 'Identifier';
    };

    isLiteralDecl = function isLiteralDecl(declaration) {
        return declaration.type === 'Literal' || declaration.type === 'NullLiteral' || declaration.type === 'StringLiteral' || declaration.type === 'BooleanLiteral' || declaration.type === 'NumericLiteral';
    };

    isObjectDecl = function isObjectDecl(declaration) {
        return declaration.type === 'ObjectExpression';
    };

    isAnnonFunctionDecl = function isAnnonFunctionDecl(declaration) {
        return (declaration.type === 'FunctionExpression' || declaration.type === 'FunctionDeclaration') && !(declaration.id && declaration.id.name);
    };

    isAnnonClassDecl = function isAnnonClassDecl(declaration) {
        return (declaration.type === 'ClassExpression' || declaration.type === 'ClassDeclaration') && !(declaration.id && declaration.id.name);
    };

    isNamedClassDecl = function isNamedClassDecl(declaration) {
        return declaration.type === 'ClassDeclaration' && declaration.id && Boolean(declaration.id.name);
    };

    transform = function transform(nodePath, name) {
        var declaration = undefined,
            id = undefined;

        declaration = nodePath.node.declaration;
        id = t.identifier(name);

        if (isAnnonFunctionDecl(declaration)) {
            declaration = t.functionExpression(null, declaration.params, declaration.body, declaration.generator);
        } else if (isAnnonClassDecl(declaration)) {
            declaration = t.classExpression(null, declaration.superClass, declaration.body, declaration.decorators || []);
        }

        nodePath.replaceWithMultiple([t.variableDeclaration('let', [t.variableDeclarator(id, declaration)]), t.exportDefaultDeclaration(id)]);
    };

    deduceName = function deduceName(pluginPass, scope) {
        var name = undefined;

        name = pluginPass && pluginPass.file && pluginPass.file.opts && pluginPass.file.opts.filename;
        name = name && _path2.default.basename(name, '.js');
        name = name && (0, _camelCase3.default)(name);

        if (!t.isValidIdentifier(name)) {
            throw Error('Invalid identifier "' + name + '".');
        }

        return resolveClashingName(name, scope);
    };

    resolveClashingName = function resolveClashingName(name, scope) {
        var index = undefined,
            resolvedName = undefined;

        index = 0;
        resolvedName = name;
        while (scope.hasBinding(resolvedName) && index < 100) {
            resolvedName = name + index++;
        }
        if (index > 100) {
            throw Error('Couldn\'t resolve clashing name "' + name + '".');
        }
        return resolvedName;
    };

    return {
        visitor: {
            ExportDefaultDeclaration: function ExportDefaultDeclaration(nodePath, pluginPass) {
                var declaration = undefined;

                declaration = nodePath.node.declaration;

                if (isIdentifierDecl(declaration) || isNamedFunctionDecl(declaration) || isLiteralDecl(declaration) || isObjectDecl(declaration) || isNamedClassDecl(declaration)) {
                    return;
                }

                transform(nodePath, deduceName(pluginPass, nodePath.scope));
            }
        }
    };
};

module.exports = exports['default'];
//# sourceMappingURL=index.js.map
